@@ -170,7 +170,7 @@ class dSturmianConfiguration(object):
     def rectangular_subword(self, window):
         r"""
         Return the rectangular subword appearing in the
-        given retangular window.
+        given rectangular window.
 
         INPUT:
 
@@ -196,7 +196,7 @@ class dSturmianConfiguration(object):
     def rectangular_subword_matrix(self, window):
         r"""
         Return the rectangular subword appearing in the
-        given retangular window (as a matrix).
+        given rectangular window (as a matrix).
 
         INPUT:
 
@@ -224,7 +224,7 @@ class dSturmianConfiguration(object):
     def rectangular_subword_tikz(self, window, node_format=None, extra_code_after=''):
         r"""
         Return the rectangular subword appearing in the
-        given retangular window (as a TikzPicture).
+        given rectangular window (as a TikzPicture).
 
         INPUT:
 
@@ -232,7 +232,7 @@ class dSturmianConfiguration(object):
         - ``node_format`` -- function or ``None``, a function giving the
           format for the matrix node at coordinate (i,j) like ``lambda
           i,j:r"{{\color{{black!60}}\symb{{{}}}}}"``.  If None, it gets
-          replaced by a function which put red at postions `(0,...,0)` and
+          replaced by a function which put red at positions `(0,...,0)` and
           `-e_i` and black elsewhere.
         - ``extra_code_after`` -- string (default: ``''``)
 
@@ -266,7 +266,7 @@ class dSturmianConfiguration(object):
             fill_color=None, extra_code_before='', extra_code_after=''):
         r"""
         Return the rectangular subword appearing in the
-        given retangular window (as a TikzPicture).
+        given rectangular window (as a TikzPicture).
 
         INPUT:
 
@@ -290,6 +290,14 @@ class dSturmianConfiguration(object):
             sage: tikz = c.rectangular_subword_discrete_plane_tikz(((0,3),(0,4)))
             sage: tikz.pdf()    # not tested
 
+        A workaround using a convention based on the ordering of the normal
+        vector was found to fix the overlaps of rhombus. I should
+        understand and clean this hack some day::
+
+            sage: c = dSturmianConfiguration((.72, .34), 0)
+            sage: tikz = c.rectangular_subword_discrete_plane_tikz(((0,3),(0,4)))
+            sage: tikz.pdf()    # not tested
+
         """
         # shift the coordinates of fill_color
         if fill_color:
@@ -306,10 +314,16 @@ class dSturmianConfiguration(object):
             shifted_fill_color = None
         # get the table
         table = self.rectangular_subword(window)
+
+        # get the good convention for drawing the rhombus
+        a1, a2 = self._normal_vector
+        convention = 'increasing' if a1 < a2 else 'decreasing'
+
         return table_to_discrete_plane_tikz(table, 
                 fill_color=shifted_fill_color,
                 extra_code_before=extra_code_before,
-                extra_code_after=extra_code_after)
+                extra_code_after=extra_code_after,
+                convention=convention)
 
     def pattern_number_occurrences(self, shape, window, avoid_border=0):
         r"""
@@ -370,6 +384,93 @@ class dSturmianConfiguration(object):
                 C[pattern] += 1
         return C
 
+    def pattern_complexity(self, shape, window, avoid_border=0, verbose=False):
+        r"""
+        Return the number of patterns having a given shape inside of a
+        rectangular window box.
+
+        INPUT:
+
+        - ``shape`` -- list, list of coordinates
+        - ``window`` -- tuple of 2-tuples
+        - ``avoid_border`` -- integer (default: 0), the size of the border
+            to avoid during the computation
+        - ``verbose`` -- bool (default: ``False``), print the theoretical
+          upper-bound
+
+        OUTPUT
+
+        integer
+
+        EXAMPLES::
+
+            sage: z = polygen(QQ, 'z')
+            sage: K = NumberField(z**2-z-1, 'phi', embedding=RR(1.6))
+            sage: phi = K.gen()
+
+        ::
+
+            sage: from slabbe import dSturmianConfiguration
+            sage: c = dSturmianConfiguration((phi^-1, phi^-2), 0)
+            sage: shape = [(0,0), (1,0), (0,1), (1,1)]
+            sage: window = ((0,10),(0,10))
+            sage: c.pattern_complexity(shape, window)
+            5
+
+        Totally irrational normal vector::
+
+            sage: c = dSturmianConfiguration((phi^-1, sqrt(2)), 0)
+            sage: shape = [(0,0), (1,0), (0,1), (1,1)]
+            sage: window = ((0,10),(0,10))
+            sage: c.pattern_complexity(shape, window)
+            8
+
+        """
+        if verbose:
+            print("upper bound is:", self.pattern_complexity_upper_bound(shape))
+        C = self.pattern_number_occurrences(shape, window, avoid_border=avoid_border)
+        return len(C)
+
+    def pattern_complexity_upper_bound(self, shape):
+        r"""
+        Return the known upper bound for the number of patterns of given
+        shape it may have.
+
+        INPUT:
+
+        - ``shape`` -- list, list of coordinates
+
+        OUTPUT
+
+        integer
+
+        EXAMPLES::
+
+            sage: z = polygen(QQ, 'z')
+            sage: K = NumberField(z**2-z-1, 'phi', embedding=RR(1.6))
+            sage: phi = K.gen()
+
+        ::
+
+            sage: from slabbe import dSturmianConfiguration
+            sage: c = dSturmianConfiguration((phi^-1, phi^-2), 0)
+            sage: shape = [(0,0), (1,0), (0,1), (1,1)]
+            sage: c.pattern_complexity_upper_bound(shape)
+            8
+
+        Totally irrational normal vector::
+
+            sage: c = dSturmianConfiguration((phi^-1, sqrt(2)), 0)
+            sage: shape = [(0,0), (1,0), (0,1), (1,1)]
+            sage: c.pattern_complexity_upper_bound(shape)
+            8
+
+        """
+        F = [vector(t) for t in ((0,0), (0,-1), (-1,0))]
+        S = [vector(t) for t in shape]
+        F_minus_S = set(tuple(f - s) for f in F for s in S)
+        return len(F_minus_S)
+
     def rectangular_subwords(self, sizes, window):
         r"""
         Return the list of rectangular subword appearing in the
@@ -529,7 +630,8 @@ def matrix_to_tikz(M, node_format=None, boundary_dash_line=False, extra_code_aft
     macros = [r'\newcommand{\symb}[1]{\mathtt{#1}}  % Symbol']
     return TikzPicture('\n'.join(lines), usetikzlibrary=usetikzlibrary, macros=macros)
 
-def table_to_discrete_plane_tikz(table, fill_color=None, extra_code_before='', extra_code_after=''):
+def table_to_discrete_plane_tikz(table, fill_color=None,
+        extra_code_before='', extra_code_after='', convention='increasing'):
     r"""
     Return a discrete plane representation of the table over alphabet `0`,
     `1` and `2`.
@@ -545,11 +647,19 @@ def table_to_discrete_plane_tikz(table, fill_color=None, extra_code_before='', e
       Rhombus of type ``a`` have color ``fill_color[a]``.
       If tuple ``(i,j)`` is in ``fill_color``, then ``fill_color[(i,j)]``
       gives the color of the rhombus at position (i,j).
+    - ``convention`` -- string, ``'decreasing'`` or ``'increasing'``, the
+      convention for drawing the rhombus according to increasing or
+      decreasing normal vectors
 
     OUTPUT:
 
     TikzPicture
 
+    .. TODO::
+
+        Understand, improve and replace the ``convention`` argument by
+        something better.
+
     EXAMPLES::
 
         sage: from slabbe.ddim_sturmian_configuration import table_to_discrete_plane_tikz
@@ -580,16 +690,27 @@ def table_to_discrete_plane_tikz(table, fill_color=None, extra_code_before='', e
     import itertools
     from slabbe import M3to2
 
+
     # faces of a unit cube
     zero = vector((0,0,0))
     e0 = vector((1,0,0))
     e1 = vector((0,1,0))
     e2 = vector((0,0,1))
-    cube_faces = {0:[zero, -e0, -e0-e1, -e1, zero],
-                  1:[zero, e2, e2-e0, -e0, zero],
-                  2:[zero, e1, e1+e2, e2, zero]}
-    label_angle = {0:90, 1:80, 2:30}
-    cone_angle = {0:(30,150), 1:(30,90), 2:(-30,90)}
+    if convention == "increasing":
+        cube_faces = {0:[zero, -e0, -e0-e1, -e1, zero],
+                      1:[zero, e2, e2-e0, -e0, zero],
+                      2:[zero, e1, e1+e2, e2, zero]}
+        label_angle = {0:90, 1:80, 2:30}
+        cone_angle = {0:(30,150), 1:(30,90), 2:(-30,90)}
+    elif convention == "decreasing":
+        cube_faces = {0:[zero, e1, e1+e2, e2, zero],
+                      1:[zero, e2, e2-e0, -e0, zero],
+                      2:[zero, -e0, -e0-e1, -e1, zero]}
+        label_angle = {0:30, 1:80, 2:90}
+        cone_angle = {0:(-30,90), 1:(30,90), 2:(30,150)}
+    else:
+        raise ValueError('convention(={}) is not a valid option'.format(convention))
+
     if fill_color is None:
         fill_color = {0:'black!10',
                       1:'black!30',
