@@ -77,10 +77,19 @@ AUTHORS:
 #                  http://www.gnu.org/licenses/
 #*****************************************************************************
 from __future__ import absolute_import, print_function
-from sage.misc.latex import have_pdflatex, have_convert, have_program
+
+from subprocess import run, PIPE, CalledProcessError
+import os
+
+from sage.misc.latex import have_pdflatex, have_convert
+try:
+    from sage.misc.latex import have_program
+except ImportError:
+    from sage.misc.sage_ostools import have_program
+
 from sage.misc.temporary_file import tmp_filename
 from sage.structure.sage_object import SageObject
-import os
+
 
 class StandaloneTex(SageObject):
     def __init__(self, content, standalone_options=None, usepackage=['amsmath'],
@@ -348,30 +357,16 @@ class StandaloneTex(SageObject):
         base, _filename_tex = os.path.split(_filename_tex)
         _filename, ext = os.path.splitext(_filename_tex)
 
-        # subprocess stuff
-        from subprocess import check_call, CalledProcessError, PIPE
-
         # running pdflatex or lualatex
         cmd = [program, '-interaction=nonstopmode', _filename_tex]
         cmd = ' '.join(cmd)
-        try:
-            check_call(cmd, shell=True, stdout=PIPE, stderr=PIPE, cwd=base)
-        except (CalledProcessError, OSError):
-            _filename_log = os.path.join(base, _filename+'.log')
-            if os.path.exists(_filename_log):
-                with open(_filename_log) as f:
-                    print(f.read())
-            else:
-                print("Error: log file was not found")
-            raise OSError("Error when running {} (see log printed above).".format(program))
-        else:
-            _filename_pdf = os.path.join(base, _filename+'.pdf')
+        run(cmd, shell=True, stdout=PIPE, stderr=PIPE, cwd=base, check=True)
+        _filename_pdf = os.path.join(base, _filename+'.pdf')
 
         # move the pdf into the good location
         if filename:
             filename = os.path.abspath(filename)
-            cmd = ['mv', _filename_pdf, filename]
-            check_call(cmd, stdout=PIPE, stderr=PIPE)
+            os.rename(_filename_pdf, filename)
             return filename
 
         # open the tmp pdf
@@ -379,7 +374,7 @@ class StandaloneTex(SageObject):
             from sage.misc.viewer import pdf_viewer
             cmd = [pdf_viewer(), _filename_pdf]
             cmd = ' '.join(cmd)
-            check_call(cmd, shell=True, cwd=base, stdout=PIPE, stderr=PIPE)
+            run(cmd, shell=True, cwd=base, stdout=PIPE, stderr=PIPE, check=True)
 
         return _filename_pdf
 
@@ -427,10 +422,6 @@ class StandaloneTex(SageObject):
                   "appear to be installed. Converting PDFLaTeX output to png "
                   "requires this program, so please install and try again. "
                   "Go to http://www.imagemagick.org to download it.")
-
-        # subprocess stuff
-        from subprocess import check_call, PIPE
-
         _filename_pdf = self.pdf(filename=None, view=False)
         _filename, ext = os.path.splitext(_filename_pdf)
         _filename_png = _filename+'.png'
@@ -440,13 +431,12 @@ class StandaloneTex(SageObject):
                '{0}x{0}'.format(density), '-trim', _filename_pdf,
                _filename_png]
         cmd = ' '.join(cmd)
-        check_call(cmd, shell=True, stdout=PIPE, stderr=PIPE)
+        run(cmd, shell=True, stdout=PIPE, stderr=PIPE, check=True)
 
         # move the png into the good location
         if filename:
             filename = os.path.abspath(filename)
-            cmd = ['mv', _filename_png, filename]
-            check_call(cmd, stdout=PIPE, stderr=PIPE)
+            os.rename(_filename_png, filename)
             return filename
 
         # open the tmp png
@@ -454,7 +444,7 @@ class StandaloneTex(SageObject):
             from sage.misc.viewer import png_viewer
             cmd = [png_viewer(), _filename_png]
             cmd = ' '.join(cmd)
-            check_call(cmd, shell=True, stdout=PIPE, stderr=PIPE)
+            run(cmd, shell=True, stdout=PIPE, stderr=PIPE, check=True)
 
         return _filename_png
 
@@ -499,9 +489,6 @@ class StandaloneTex(SageObject):
                     "Install it for example with ``brew install pdf2svg``"
                     " or ``apt-get install pdf2svg``.")
 
-        # subprocess stuff
-        from subprocess import check_call, PIPE
-
         _filename_pdf = self.pdf(filename=None, view=False)
         _filename, ext = os.path.splitext(_filename_pdf)
         _filename_svg = _filename+'.svg'
@@ -509,13 +496,12 @@ class StandaloneTex(SageObject):
         # convert to svg
         cmd = ['pdf2svg', _filename_pdf, _filename_svg]
         cmd = ' '.join(cmd)
-        check_call(cmd, shell=True, stdout=PIPE, stderr=PIPE)
+        run(cmd, shell=True, stdout=PIPE, stderr=PIPE, check=True)
 
         # move the svg into the good location
         if filename:
             filename = os.path.abspath(filename)
-            cmd = ['mv', _filename_svg, filename]
-            check_call(cmd, stdout=PIPE, stderr=PIPE)
+            os.rename(_filename_svg, filename)
             return filename
 
         # open the tmp svg
@@ -523,7 +509,7 @@ class StandaloneTex(SageObject):
             from sage.misc.viewer import browser
             cmd = [browser(), _filename_svg]
             cmd = ' '.join(cmd)
-            check_call(cmd, shell=True, stdout=PIPE, stderr=PIPE)
+            run(cmd, shell=True, stdout=PIPE, stderr=PIPE, check=True)
 
         return _filename_svg
 
@@ -623,6 +609,56 @@ class TikzPicture(StandaloneTex):
         ....:        usetikzlibrary=tikzlib, macros=macros)
         sage: _ = t.pdf(view=False)   # long time (2s)
     """
+    @classmethod
+    def from_dot_string(cls, dotdata, prog='dot'):
+        r"""
+        Convert a graph to a tikzpicture using graphviz and dot2tex.
+
+        .. NOTE::
+
+            Prerequisite: dot2tex optional Sage package and graphviz must be
+            installed.
+
+        INPUT:
+
+        - ``dotdata`` -- dot format string
+        - ``prog`` -- string (default: ``'dot'``) the program used for the
+          layout corresponding to one of the software of the graphviz
+          suite: 'dot', 'neato', 'twopi', 'circo' or 'fdp'.
+
+        EXAMPLES::
+
+            sage: from slabbe import TikzPicture
+            sage: G = graphs.PetersenGraph()
+            sage: dotdata = G.graphviz_string()
+            sage: tikz = TikzPicture.from_dot_string(dotdata) # optional dot2tex # long time (3s)
+            sage: _ = tikz.pdf()      # not tested
+            sage: dotdata = G.graphviz_string(labels='latex')
+            sage: tikz = TikzPicture.from_dot_string(dotdata) # optional dot2tex # long time (3s)
+            sage: _ = tikz.pdf()      # not tested
+
+        ::
+
+            sage: W = CoxeterGroup(["A",2])
+            sage: G = W.cayley_graph()
+            sage: dotdata = G.graphviz_string()
+            sage: tikz = TikzPicture.from_dot_string(dotdata) # optional dot2tex # long time (3s)
+            sage: _ = tikz.pdf()      # not tested
+            sage: dotdata = G.graphviz_string(labels='latex')
+            sage: tikz = TikzPicture.from_dot_string(dotdata) # optional dot2tex # long time (3s)
+            sage: _ = tikz.pdf()      # not tested
+
+        """
+        import dot2tex
+        tikz = dot2tex.dot2tex(dotdata,
+                               format='tikz',
+                               autosize=True,
+                               crop=True,
+                               figonly='True',
+                               prog=prog).strip()
+        return TikzPicture(tikz, standalone_options=["border=4mm"],
+                           usetikzlibrary=['shapes'])
+
     @classmethod
     def from_graph(cls, graph, merge_multiedges=True,
             merge_label_function=tuple, **kwds):
@@ -709,6 +745,29 @@ class TikzPicture(StandaloneTex):
             ....:                               subgraph_clusters=C)
             sage: _ = tikz.pdf()      # not tested
 
+        An example coming from ``graphviz_string`` documentation in SageMath::
+
+            sage: f(x) = -1 / x
+            sage: g(x) = 1 / (x + 1)
+            sage: G = DiGraph()
+            sage: G.add_edges((i, f(i), f) for i in (1, 2, 1/2, 1/4))
+            sage: G.add_edges((i, g(i), g) for i in (1, 2, 1/2, 1/4))
+            sage: t = TikzPicture.from_graph(G)
+            sage: _ = tikz.pdf()      # not tested
+            sage: def edge_options(data):
+            ....:     u, v, label = data
+            ....:     options = {"color": {f: "red", g: "blue"}[label]}
+            ....:     if (u,v) == (1/2, -2): options["label"]       = "coucou"; options["label_style"] = "string"
+            ....:     if (u,v) == (1/2,2/3): options["dot"]         = "x=1,y=2"
+            ....:     if (u,v) == (1,   -1): options["label_style"] = "latex"
+            ....:     if (u,v) == (1,  1/2): options["edge_string"] = "->"
+            ....:     if (u,v) == (1/2,  1): options["backward"]    = True
+            ....:     return options
+            sage: t = TikzPicture.from_graph(G, edge_options=edge_options)
+            sage: _ = tikz.pdf()      # not tested
+
+        .. TODO:: improve the previous example
+
         """
         from sage.misc.latex import have_pdflatex
         assert have_pdflatex(), "pdflatex does not seem to be installed"
